<?php

if (!defined('_PS_VERSION_'))
	exit;
	
class Payanyway extends PaymentModule
{
	private $host_url;
	
	public function __construct()
	{
		$this->name = 'payanyway';
		$this->tab = 'payments_gateways';
		$this->version = '1.1';
		$this->author = 'PayAnyWay';

		parent::__construct();

		$this->displayName = 'PayAnyWay';
		$this->description = $this->l('Receive payment with PayAnyWay');

		$this->host_url = (Configuration::get('PS_SHOP_DOMAIN') ? Configuration::get('PS_SHOP_DOMAIN') : $_SERVER['HTTP_HOST']);

		$this->_payment_systems = array(
			'payanyway'	=> array( 'title' => $this->l('PayAnyWay'), 'unitId' => 0, 'accountId' => 0, 'isInvoice' => 0 ),
			'alfaclick'	=> array( 'title' => $this->l('Internet-bank "Alfa-Click"'), 'unitId' => 587412, 'accountId' => 0, 'isInvoice' => 0 ),
			'dengimail'	=> array( 'title' => $this->l('Money@mail.ru'), 'unitId' => 545234, 'accountId' => 0, 'isInvoice' => 0 ),
			'faktura'	=> array( 'title' => $this->l('Faktura'), 'unitId' => 609111, 'accountId' => 0, 'isInvoice' => 0 ),
			'moneymail'	=> array( 'title' => $this->l('MoneyMail'), 'unitId' => 1038, 'accountId' => 29, 'isInvoice' => 0 ),
			'moneta'	=> array( 'title' => $this->l('Moneta.ru'), 'unitId' => 1015, 'accountId' => 94, 'isInvoice' => 0 ),
			'plastic'	=> array( 'title' => $this->l('VISA, Master Card'), 'unitId' => 'card', 'accountId' => 159, 'isInvoice' => 0 ),
			'psb'		=> array( 'title' => $this->l('Internet-bank "Promsvyazbank"'), 'unitId' => 661709, 'accountId' => 0, 'isInvoice' => 0 ),
			'walletone'	=> array( 'title' => $this->l('Wallet One'), 'unitId' => 310212, 'accountId' => 153, 'isInvoice' => 0 ),
			'webmoney'	=> array( 'title' => $this->l('WebMoney'), 'unitId' => 1017, 'isInvoice' => 0 ),
			'yandex'	=> array( 'title' => $this->l('Yandex.Money'), 'unitId' => 1020, 'accountId' => 16, 'isInvoice' => 0 ),
			'qiwi'		=> array( 'title' => $this->l('QIWI Wallet'), 'unitId' => 822360, 'accountId' => 0, 'isInvoice' => 0 ),
		);
		if (!extension_loaded('soap'))
			$this->warning .= $this->l('To work properly the module need the Soap extension to be installed.');
		else
			$this->_payment_systems = array_merge($this->_payment_systems, array(
				'banktransfer'	=> array( 'title' => $this->l('Bank transfer'), 'unitId' => 705000, 'accountId' => 75983431, 'isInvoice' => 1 ),
				'ciberpay'		=> array( 'title' => $this->l('CiberPay'), 'unitId' => 489755, 'accountId' => 19357960, 'isInvoice' => 1 ),
				'comepay'		=> array( 'title' => $this->l('Comepay'), 'unitId' => 228820, 'accountId' => 47654606, 'isInvoice' => 1 ),
				'contact'		=> array( 'title' => $this->l('Contact'), 'unitId' => 1028, 'accountId' => 26, 'isInvoice' => 1 ),
				'elecsnet'		=> array( 'title' => $this->l('Elecsnet'), 'unitId' => 232821, 'accountId' => 10496472, 'isInvoice' => 1 ),
				'euroset'		=> array( 'title' => $this->l('Euroset, Svyaznoi'), 'unitId' => 248362, 'accountId' => 136, 'isInvoice' => 1 ),
				'forward'		=> array( 'title' => $this->l('Forward Mobile'), 'unitId' => 83046, 'accountId' => 116, 'isInvoice' => 1 ),
				'gorod'			=> array( 'title' => $this->l('Federal System GOROD'), 'unitId' => 426904, 'accountId' => 152, 'isInvoice' => 1 ),
				'mcb'			=> array( 'title' => $this->l('MosCreditBank'), 'unitId' => 295339, 'accountId' => 143, 'isInvoice' => 1 ),
				'novoplat'		=> array( 'title' => $this->l('NovoPlat'), 'unitId' => 281129, 'accountId' => 80314912, 'isInvoice' => 1 ),
				'platika'		=> array( 'title' => $this->l('PLATiKA'), 'unitId' => 226272, 'accountId' => 15662295, 'isInvoice' => 1 ),
				'post'			=> array( 'title' => $this->l('Russian Post Transfer'), 'unitId' => 1029, 'accountId' => 15, 'isInvoice' => 1 ),
				'sberbank'		=> array( 'title' => $this->l('Sberbank'), 'unitId' => 510801, 'accountId' => 160, 'isInvoice' => 1 ),
			));
		
	}

	public function install()
	{
		if (!parent::install() OR 
			!$this->registerHook('payment') OR 
			!$this->registerHook('orderConfirmation'))
			return false;
		Configuration::updateValue('PAYANYWAY_PAYMENT_URL', 'www.payanyway.ru');
		Configuration::updateValue('PAYANYWAY_MNT_ID', '');
		Configuration::updateValue('PAYANYWAY_MNT_DATAINTEGRITY_CODE', '');
		Configuration::updateValue('PAYANYWAY_MNT_TEST_MODE', '0');
		Configuration::updateValue('PAYANYWAY_LOGIN', '');
		Configuration::updateValue('PAYANYWAY_PASSWORD', '');
		Configuration::updateValue('PAYANYWAY_PAYMENT_SYSTEMS', '');
		
		return true;
	}

	public function uninstall()
	{
		Configuration::deleteByName('PAYANYWAY_PAYMENT_URL');
		Configuration::deleteByName('PAYANYWAY_MNT_ID');
		Configuration::deleteByName('PAYANYWAY_MNT_DATAINTEGRITY_CODE');
		Configuration::deleteByName('PAYANYWAY_MNT_TEST_MODE');
		Configuration::deleteByName('PAYANYWAY_LOGIN');
		Configuration::deleteByName('PAYANYWAY_PASSWORD');
		Configuration::deleteByName('PAYANYWAY_PAYMENT_SYSTEMS');

		return parent::uninstall();
	}

	public function hookOrderConfirmation($params)
	{
		global $smarty;

		if ($params['objOrder']->module != $this->name) 
			return;

		$status = Tools::getvalue('status');
		if ($status == 'success')
		{
			if ($params['objOrder']->getCurrentState() != Configuration::get('PS_OS_PAYMENT'))
				$smarty->assign(array('status' => 'waiting', 'id_order' => intval($params['objOrder']->id)));
			else
				$smarty->assign(array('status' => 'success', 'id_order' => intval($params['objOrder']->id)));
		}
		elseif ($status == 'fail')
		{
			$params['objOrder']->setCurrentState(Configuration::get('PS_OS_CANCELED'));
			$smarty->assign('status', 'failed');
		}
		elseif ($status == 'invoice')
		{
			$smarty->assign(array('status' => 'invoice', 'invoice' => $this->_invoice($params)));
		}

		return $this->display(__FILE__, 'invoice.tpl'); 
	}
	
	private function _invoice($params)
	{
		global $smarty;
		
		$payanyway_server = Configuration::get('PAYANYWAY_PAYMENT_URL');
		$login = Configuration::get('PAYANYWAY_LOGIN');
		$password = Configuration::get('PAYANYWAY_PASSWORD');

		require_once (dirname(__FILE__).'/MonetaAPI/MonetaWebService.php');
		switch ($payanyway_server) {
			case 'www.payanyway.ru':
				$service = new MonetaWebService("https://www.moneta.ru/services.wsdl", $login, $password);
				break;
			case 'demo.moneta.ru':
				$service = new MonetaWebService("https://demo.moneta.ru/services.wsdl", $login, $password);
				break;
		}

		try
		{
			$totalAmount = Tools::getvalue('MNT_AMOUNT')." ".Tools::getvalue('MNT_CURRENCY_CODE');
			$fee = "-";

			// запрос стоимости и комиссии
			$accountId = Tools::getvalue('paymentSystem_accountId', null);
			if (!is_null($accountId)) {
				$transactionRequestType = new MonetaForecastTransactionRequest();
				$transactionRequestType->payer = Tools::getvalue('paymentSystem_accountId');
				$transactionRequestType->payee = Tools::getvalue('MNT_ID');
				$transactionRequestType->amount = Tools::getvalue('MNT_AMOUNT');
				$transactionRequestType->clientTransaction = Tools::getvalue('MNT_TRANSACTION_ID');
				$forecast = $service->ForecastTransaction($transactionRequestType);
				$totalAmount = number_format($forecast->payerAmount,2,'.','')." ".$forecast->payerCurrency;
				$fee = number_format($forecast->payerFee,2,'.','')." ".$forecast->payerCurrency;
			}
			
			// получить данные счета
			$request = new MonetaInvoiceRequest();
			if (!is_null($accountId))
				$request->payer = $accountId;
			$request->payee = Tools::getvalue('MNT_ID');
			$request->amount = Tools::getvalue('MNT_AMOUNT');
			$request->clientTransaction = Tools::getvalue('MNT_TRANSACTION_ID');

			$operationInfo = new MonetaOperationInfo();
			if (Tools::getvalue('payment_system') == 'post')
			{
				$a = new MonetaKeyValueAttribute();
				$a->key = 'mailofrussiaindex';
				$a->value = Tools::getvalue('additionalParameters_mailofrussiaSenderIndex');
				$operationInfo->addAttribute($a);
				$a1 = new MonetaKeyValueAttribute();
				$a1->key = 'mailofrussiaregion';
				$a1->value = Tools::getvalue('additionalParameters_mailofrussiaSenderRegion');
				$operationInfo->addAttribute($a1);
				$a2 = new MonetaKeyValueAttribute();
				$a2->key = 'mailofrussiaaddress';
				$a2->value = Tools::getvalue('additionalParameters_mailofrussiaSenderAddress');
				$operationInfo->addAttribute($a2);
				$a3 = new MonetaKeyValueAttribute();
				$a3->key = 'mailofrussianame';
				$a3->value = Tools::getvalue('additionalParameters_mailofrussiaSenderName');
				$operationInfo->addAttribute($a3);
			}
			elseif (Tools::getvalue('payment_system') == 'euroset')
			{
				$a1 = new MonetaKeyValueAttribute();
				$a1->key = 'rapidamphone';
				$a1->value = Tools::getvalue('additionalParameters_rapidaPhone');
				$operationInfo->addAttribute($a1);
			}
			$request->operationInfo = $operationInfo;
			
			$response = $service->Invoice($request);
			if (Tools::getvalue('payment_system') == 'euroset')
			{
				$response1 = $service->GetOperationDetailsById($response->transaction);
				foreach ($response1->operation->attribute as $attr)
				{
					if ($attr->key == 'rapidatid')
					{
						$transaction_id = $attr->value;
					}
				}
			}
			else
			{
				$transaction_id = $response->transaction;
			}

			$invoice->status = $response->status;
			$invoice->payanyway_server = $payanyway_server;
			$invoice->transaction = str_pad($transaction_id, 10, "0", STR_PAD_LEFT);
			$invoice->system = Tools::getvalue('payment_system');
			$invoice->unitId = Tools::getvalue('paymentSystem_unitId');
			$invoice->amount = $totalAmount;
			$invoice->fee = $fee;
		}
		catch (Exception $e)
		{
			$invoice->status = 'FAILED';
			$invoice->error_message = $e->getMessage();
		}
		return $invoice;
	}
	
	public function getContent()
	{
		$output = '<h2>'.$this->displayName.'</h2>';
		
		if (Tools::isSubmit('submitModule'))
		{
			Configuration::updateValue('PAYANYWAY_PAYMENT_URL', Tools::getvalue('payanyway_payment_url'));
			Configuration::updateValue('PAYANYWAY_MNT_ID', Tools::getvalue('payanyway_mnt_id'));
			Configuration::updateValue('PAYANYWAY_MNT_DATAINTEGRITY_CODE', Tools::getvalue('payanyway_mnt_dataintegrity_code'));
			Configuration::updateValue('PAYANYWAY_MNT_TEST_MODE', Tools::getvalue('payanyway_mnt_test_mode'));
			Configuration::updateValue('PAYANYWAY_LOGIN', Tools::getvalue('payanyway_login'));
			Configuration::updateValue('PAYANYWAY_PASSWORD', Tools::getvalue('payanyway_password'));
			Configuration::updateValue('PAYANYWAY_PAYMENT_SYSTEMS', serialize(Tools::getvalue('payment_systems')));
		
			echo $this->displayConfirmation($this->l('Configuration updated'));
		}

		$output .= '
		<fieldset>
			<a href="http://www.payanyway.ru/" target="_blank"  style="float:left;margin-right:15px;"><img src="../modules/payanyway/payanyway.png" alt="PayAnyWay" /></a>
			<b>'.$this->l('This module allows you to accept payments with PayAnyWay.').'</b><br/><br/>
			<a target=\'_blank\' style=\'text-decoration:underline;\' href=\'http://code.google.com/p/payanyway/wiki/prestashop_type3\'>'.$this->l('Follow instructions to configure PayAnyWay in Prestashop.').'</a>
			<br />
		</fieldset><br />
		<form action="'.Tools::htmlentitiesutf8($_SERVER['REQUEST_URI']).'" method="post">
			<fieldset>
				<legend><img src="../img/admin/contact.gif" alt="" />'.$this->l('Settings').'</legend>
				<label for="payanyway_payment_url">'.$this->l('Payment URL').'</label>
				<div class="margin-form">
				<select id="payanyway_payment_url" name="payanyway_payment_url">';
				$payment_urls = array('www.payanyway.ru', 'demo.moneta.ru');
				foreach ($payment_urls as $item)
				{
					$selected = (Configuration::get('PAYANYWAY_PAYMENT_URL') == $item) ? 'selected="selected"' : '';
					$output .= '<option value="'.$item.'" '.$selected.'>'.$item.'</option>';
				}
			$output .='
				</select>
				</div>
				<label for="payanyway_mnt_id">'.$this->l('Account number').'</label>
				<div class="margin-form"><input type="text" size="20" id="payanyway_mnt_id" name="payanyway_mnt_id" value="'.Configuration::get('PAYANYWAY_MNT_ID').'" /></div>
				<label for="payanyway_mnt_dataintegrity_code">'.$this->l('Data integrity code').'</label>
				<div class="margin-form"><input type="text" size="20" id="payanyway_mnt_dataintegrity_code" name="payanyway_mnt_dataintegrity_code" value="'.Configuration::get('PAYANYWAY_MNT_DATAINTEGRITY_CODE').'" /></div>
				<label for="payanyway_mnt_test_mode">'.$this->l('Mode:').'</label>
				<div class="margin-form" id="payanyway_mnt_test">
					<input type="radio" name="payanyway_mnt_test_mode" value="0" style="vertical-align: middle;" '.(!Tools::getValue('payanyway_mnt_test_mode', Configuration::get('PAYANYWAY_MNT_TEST_MODE')) ? 'checked="checked"' : '').' />
					<span style="color: #080;">'.$this->l('Production').'</span>
					<input type="radio" name="payanyway_mnt_test_mode" value="1" style="vertical-align: middle;" '.(Tools::getValue('payanyway_mnt_test_mode', Configuration::get('PAYANYWAY_MNT_TEST_MODE')) ? 'checked="checked"' : '').' />
					<span style="color: #900;">'.$this->l('Test').'</span>
				</div>
				<label for="payanyway_login">'.$this->l('Login in PayAnyWay').'</label>
				<div class="margin-form"><input type="text" size="20" id="payanyway_login" name="payanyway_login" value="'.Configuration::get('PAYANYWAY_LOGIN').'" /></div>
				<label for="payanyway_password">'.$this->l('Password in PayAnyWay').'</label>
				<div class="margin-form"><input type="text" size="20" id="payanyway_password" name="payanyway_password" value="'.Configuration::get('PAYANYWAY_PASSWORD').'" /></div>
					
				<label for="payment_systems">'.$this->l('Payment systems').'</label>
				<div class="margin-form" id="payanyway_systems">';
			
			$payment_systems = unserialize(Configuration::get('PAYANYWAY_PAYMENT_SYSTEMS'));
			foreach ($this->_payment_systems as $key => $value)
			{
				$checked = (in_array($key, $payment_systems)) ? 'checked="checked"' : '';
				$output .= '<input type="checkbox" id="'.$key.'" name="payment_systems[]" value="'.$key.'" '.$checked.' />
							<span>'.$value['title'].'</span><br />';
			}
			$output .=	'
				</div>
				
				<br /><center><input type="submit" name="submitModule" value="'.$this->l('Update settings').'" class="button" /></center>
			</fieldset>
		</form>';
		
		return $output;
	}

	public function hookPayment($params)
	{
		if (!$this->active)
			return ;
		if (!$this->_checkCurrency($params['cart']))
			return ;
		
		global $cookie, $smarty;
		
		$payment_systems = unserialize(Configuration::get('PAYANYWAY_PAYMENT_SYSTEMS'));
		$payment_systems = array_intersect_key($this->_payment_systems,array_flip($payment_systems));

			$customer = new Customer((int)$cookie->id_customer);
			$currency = new Currency((int)($params['cart']->id_currency));
			
			$amount = number_format($params['cart']->getOrderTotal(true, 3), 2, '.', '');

			$mnt_signature = md5(Configuration::get('PAYANYWAY_MNT_ID').(int)$params['cart']->id.$amount.$currency->iso_code.Configuration::get('PAYANYWAY_MNT_TEST_MODE').Configuration::get('PAYANYWAY_MNT_DATAINTEGRITY_CODE'));
			
			$payanywayParams = array();
			$payanywayParams['MNT_ID'] = Configuration::get('PAYANYWAY_MNT_ID');
			$payanywayParams['MNT_TRANSACTION_ID'] = (int)$params['cart']->id;
			$payanywayParams['MNT_TEST_MODE'] = Configuration::get('PAYANYWAY_MNT_TEST_MODE');
			$payanywayParams['MNT_CURRENCY_CODE'] = $currency->iso_code;
			$payanywayParams['MNT_AMOUNT'] = $amount;
			$payanywayParams['MNT_SIGNATURE'] = $mnt_signature;
			$payanywayParams['MNT_SUCCESS_URL'] = 'http://'.$this->host_url.__PS_BASE_URI__.'order-confirmation.php?id_cart='.(int)($params['cart']->id).'&id_module='.(int)($this->id).'&key='.$customer->secure_key.'&status=success';
			$payanywayParams['MNT_FAIL_URL'] = 'http://'.$this->host_url.__PS_BASE_URI__.'order-confirmation.php?id_cart='.(int)($params['cart']->id).'&id_module='.(int)($this->id).'&key='.$customer->secure_key.'&status=fail';
			$payanywayParams['followup'] = 'true';
			$payanywayParams['javascriptEnabled'] = 'true';
			
			$invoiceUrl = 'http://'.$this->host_url.__PS_BASE_URI__.'order-confirmation.php?id_cart='.(int)($params['cart']->id).'&id_module='.(int)($this->id).'&key='.$customer->secure_key.'&status=invoice&';
						
			$smarty->assign('params', $payanywayParams);
			$smarty->assign('invoice_url', $invoiceUrl);
			$smarty->assign('payment_url', Configuration::get('PAYANYWAY_PAYMENT_URL'));

		$smarty->assign('payment_systems', $payment_systems);
		
		return $this->display(__FILE__, 'payment.tpl');
	}
	
	private function _checkCurrency($cart)
	{
		$currency_order = new Currency((int)($cart->id_currency));
		$currencies_module = $this->getCurrency((int)$cart->id_currency);
		$currency_default = Configuration::get('PS_CURRENCY_DEFAULT');
		
		if (is_array($currencies_module))
			foreach ($currencies_module AS $currency_module)
				if ($currency_order->id == $currency_module['id_currency'])
					return true;
		return false;
	}
}
?>
